﻿-- Talent Sets
-- by Jadya
-- EU-Well of Eternity

local format, tremove = string.format, table.remove

local addonName, addonTable = ...
local L = addonTable.L

local icon_filenames = {}
local max_talent_sets = 15
local numTiers = 7
local numColumns = 3

local str_no_equipment = "|cffE96969"..NONE.."|r"
local equipment_cache, equipment_menu_table, equipment_menu_frame, t_menu_close, t_menu_empty
local spec, sets
local tt, check, learn = "talents", GetTalentInfo, LearnTalent

local function getSetByName(name)
 for k, v in pairs(sets) do
  if v.name == name then return k end
 end
end

local function checkEquipmentSets()
 local name, found, needUpdate
 for _,v in pairs(sets) do
  if v.equipment then
   found = false
   for i = 1, GetNumEquipmentSets() do
    name = GetEquipmentSetInfo(i)
    if v.equipment == name then
     found = true
     break
    end
   end
   
   if not found then
    print("|cff00ffffTalent Set Manager|r - "..format(L["equipment_not_found"], "|cff00ff00"..v.equipment.."|r", "|cff00ff00"..v.name.."|r"))
    v.equipment = nil
    needUpdate = true
   end
  end
 end
 
 if needUpdate then
  TalentSetList_Update()
 end
end

local function specChanged()
 if not TalentSetManager_Saves[tt][spec] then
  TalentSetManager_Saves[tt][spec] = {}
 end
 sets = TalentSetManager_Saves[tt][spec]
 if TalentSetsList then
  TalentSetsList.selectedSetName = nil
  TalentSetList_Update()
 end
 checkEquipmentSets()
end

local function modifyTalentSet(origName, name, texture)
 local i = getSetByName(origName)
 
 if not i then return end
 
 sets[i].name = name
 if texture then
  sets[i].texture = type(texture) == "number" and texture or ("Interface\\Icons\\"..texture)
 end
end

local function saveTalentSet(name, texture)
 local i = getSetByName(name)

 if not i then i = #sets + 1 end
 
 sets[i] = sets[i] or {}

 sets[i].name = name

 if texture then
  sets[i].texture = type(texture) == "number" and texture or ("Interface\\Icons\\"..texture)
 end

 for j = 1, numTiers do
  sets[i][j] = nil
  for c = 1, numColumns do
   local _, _, _, selected = check(j, c, 1)
   if selected then
    sets[i][j] = c
   end
  end
 end
end

local function deleteTalentSet(name)
 local id = getSetByName(name)

 if not sets[id] then return end
 
 tremove(sets, id)
 TalentSetsList.selectedSetName = nil
 TalentSetList_Update()
end

StaticPopupDialogs["TS_CONFIRM_SAVE_TALENT_SET"] = {
	text = L["confirm_save_set"],
	button1 = YES,
	button2 = NO,
	OnAccept = function (self) saveTalentSet(self.data) TalentSetList_Update() end,
	OnCancel = function (self) end,
	OnHide = function (self) self.data = nil end,
	hideOnEscape = 1,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
}

StaticPopupDialogs["TS_CONFIRM_DELETE_TALENT_SET"] = {
	text = L["confirm_delete_set"],
	button1 = YES,
	button2 = NO,
	OnAccept = function (self) deleteTalentSet(self.data) end,
	OnCancel = function (self) end,
	hideOnEscape = 1,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
}

StaticPopupDialogs["TS_CONFIRM_OVERWRITE_TALENT_SET"] = {
	text = L["confirm_overwrite_set"],
	button1 = YES,
	button2 = NO,
	OnAccept = function(self) saveTalentSet(self.data, self.selectedIcon) TalentSetsDialogPopup:Hide() TalentSetList_Update() end,
	OnCancel = function(self) end,
	OnHide = function(self) self.data = nil self.selectedIcon = nil end,
	hideOnEscape = 1,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
}

local function isEquipped(i)
 if not sets[i] then return end
 
 for j = 1, numTiers do
  if sets[i][j] then
   local id, _, _, selected = check(j, sets[i][j], 1)
   if not selected then
    return
   end
  end
 end
 return true
end

local function useEquipmentSet(eq)
 if not eq then return end

 if EquipmentSetContainsLockedItems(eq) or UnitCastingInfo("player") then
  UIErrorsFrame:AddMessage(ERR_CLIENT_LOCKED_OUT, 1.0, 0.1, 0.1, 1.0)
  return
 end

 UseEquipmentSet(eq)
end

local function learnTalents(i)
 if not sets[i] --[[or isEquipped(i)]] then return end
 
 for j = 1, numTiers do
  if sets[i][j] then
   local id = check(j, sets[i][j], 1)
   if id then
    learn(id)
   end
  end
 end

 useEquipmentSet(sets[i].equipment)
end

function TalentSetList_Update()
 if not TalentSetsList then return end

 local name = TalentSetsList.selectedSetName

 if name then
  local i = getSetByName(name)
  if isEquipped(i) then
   TalentSetsListSaveSet:Disable()
   TalentSetsListEquipSet:Disable()
  else
   TalentSetsListSaveSet:Enable()
   TalentSetsListEquipSet:Enable()
  end
 else
  TalentSetsListSaveSet:Disable()
  TalentSetsListEquipSet:Disable()
 end

 local numSets = #sets
 local numRows = numSets
 if numSets < max_talent_sets then
  numRows = numRows + 1  -- "Add New Set" button
 end

 HybridScrollFrame_Update(TalentSetsList, numRows * 44 + TalentSetsListEquipSet:GetHeight() + 20, TalentSetsList:GetHeight())
 
 local scrollOffset = HybridScrollFrame_GetOffset(TalentSetsList)
 local buttons = TalentSetsList.buttons
 local selectedName = TalentSetsList.selectedSetName

 local id, texture, button
 for i = 1, #buttons do
  if (i+scrollOffset <= numRows) then
   button = buttons[i]
   buttons[i]:Show()
   button:Enable()
   id = i + scrollOffset
 
   if id <= numSets then
    button.name = sets[id].name
    button.hasEquipment = sets[id].equipment
    button.text:SetText(sets[id].name)
    button.text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
                
    if sets[id].texture then
     button.icon:SetTexture(sets[id].texture)
    else
     button.icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
    end

    if selectedName and button.name == selectedName then
     button.SelectedBar:Show()
    else
     button.SelectedBar:Hide()
    end
    
    if isEquipped(id) then
     button.Check:Show()
    else
     button.Check:Hide()
    end

    button.icon:SetSize(36, 36)
    button.icon:SetPoint("LEFT", 4, 0)
   else
    -- This is the Add New button
    button.name = nil
    button.hasEquipment = nil
    button.text:SetText(PAPERDOLL_NEWEQUIPMENTSET)
    button.text:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b)
    button.icon:SetTexture("Interface\\PaperDollInfoFrame\\Character-Plus")
    button.icon:SetSize(30, 30)
    button.icon:SetPoint("LEFT", 7, 0)
    button.Check:Hide()
    button.SelectedBar:Hide()
   end
    
   if (i+scrollOffset) == 1 then
    buttons[i].BgTop:Show()
    buttons[i].BgMiddle:SetPoint("TOP", buttons[i].BgTop, "BOTTOM")
   else
    buttons[i].BgTop:Hide()
    buttons[i].BgMiddle:SetPoint("TOP")
   end
    
   if ((i+scrollOffset) == numRows) then
    buttons[i].BgBottom:Show()
    buttons[i].BgMiddle:SetPoint("BOTTOM", buttons[i].BgBottom, "TOP")
   else
    buttons[i].BgBottom:Hide()
    buttons[i].BgMiddle:SetPoint("BOTTOM")
   end
    
   if ((i+scrollOffset)%2 == 0) then
    buttons[i].Stripe:SetColorTexture(0.9, 0.9, 1)
    buttons[i].Stripe:SetAlpha(0.1)
    buttons[i].Stripe:Show()
   else
    buttons[i].Stripe:Hide()
   end
  else
   buttons[i]:Hide()
  end
 end
end

local function setTexture(f, t)
 if f.tx ~= t then
  f:SetTexture(t)
  f.tx = t
 end
end

local last_t = 0
local function listBox_update(f, elapsed)
 for i = 1, #f.buttons do
  local button = f.buttons[i]
  if button:IsMouseOver() then
   if button.name and button.name ~= "" then
  	button.DeleteButton:Show()
  	button.EditButton:Show()
    button.EquipmentButton:Show()
   else
    button.DeleteButton:Hide()
    button.EditButton:Hide()
    button.EquipmentButton:Hide()
   end
   button.HighlightBar:Show()
  else
   button.DeleteButton:Hide()
   button.EditButton:Hide()
   button.HighlightBar:Hide()
   if button.hasEquipment then
    button.EquipmentButton:Show()
    setTexture(button.EquipmentButton.texture, "Interface\\Minimap\\minimap_shield_elite")
   else
    button.EquipmentButton:Hide()
    setTexture(button.EquipmentButton.texture, "Interface\\Minimap\\minimap_shield_normal")
   end
  end
 end
 
 if f.queuedUpdate then
  last_t = last_t + elapsed
  if last_t > 0.5 then
   TalentSetList_Update()
   f.queuedUpdate = false
   last_t = 0
  end
 end
end

function TalentSetList_OnEvent(self, event, ...)
 -- PLAYER_PVP_TALENT_UPDATE -  looks like PLAYER_TALENT_UPDATE is also fired when changing pvp talents
 if event == "PLAYER_TALENT_UPDATE" then
  TalentSetList_Update()
 end
end

function TalentSetList_OnHide(self)
 TalentSetsDialogPopup:Hide()
 StaticPopup_Hide("TS_CONFIRM_DELETE_TALENT_SET")
 StaticPopup_Hide("TS_CONFIRM_OVERWRITE_TALENT_SET")
 equipment_menu_frame:Hide()
end

function TalentSetListButton_Generic_OnLeave(f)
 f.texture:SetAlpha(0.5)
 GameTooltip_Hide()
end

----- equipment menu -----
local function equipment_menuitem_onClick(f)
 local id = equipment_menu_frame.editingID
 
 if not id then return end

 sets[id].equipment = f.value ~= str_no_equipment and f.value or nil
 TalentSetList_Update()
end

local function get()
 return tremove(equipment_cache, 1) or {}
end

local function dispose(t)
 equipment_cache[#equipment_cache + 1] = t
end

function TalentSetListButton_Equipment_OnClick(f)
 local menu_index = 5

 local id = getSetByName(f:GetParent().name)

 for i = 5, #equipment_menu_table do
  if equipment_menu_table[i].TSMdynamic then
   dispose(equipment_menu_table[i])
  end
  equipment_menu_table[i] = nil
 end

 -- 'None' entry
 equipment_menu_table[4].checked = sets[id].equipment == nil
 
 local i = 1
 while i <= GetNumEquipmentSets() do
  local t = get()
  found = t.text == sets[id].equipment
  t.text = GetEquipmentSetInfo(i)
  t.value = t.text
  t.func = equipment_menuitem_onClick
  t.checked = t.text == sets[id].equipment
  t.TSMdynamic = true
  
  equipment_menu_table[menu_index] = t
  i = i + 1
  menu_index = menu_index + 1
 end
 
 equipment_menu_table[menu_index] = t_menu_empty
 equipment_menu_table[menu_index + 1] = t_menu_close
 
 equipment_menu_frame.editingID = id
 EasyMenu(equipment_menu_table, equipment_menu_frame, "cursor", 0 , 0, "MENU")
end
---------------------

function TalentSetListButton_Equipment_OnEnter(f)
 local eq
 local id = getSetByName(f:GetParent().name)
 eq = sets[id].equipment
 
 f.texture:SetAlpha(1.0)
 GameTooltip:SetOwner(f, "ANCHOR_RIGHT")
 GameTooltip:AddLine(L["link_equipment"])
 GameTooltip:AddLine(format(L["current_equipment"], eq and ("|cff00ff00"..eq.."|r") or str_no_equipment))
 GameTooltip:Show()
 --[[
 if eq then
  GameTooltip:SetEquipmentSet(eq)
 end
 ]]
end

function TalentSetListButton_Delete_OnClick(f)
 local id = getSetByName(f:GetParent().name)
 if not id or not sets[id] then return end

 local dialog = StaticPopup_Show("TS_CONFIRM_DELETE_TALENT_SET", sets[id].name)
 if dialog then
  dialog.data = sets[id].name
 else
  UIErrorsFrame:AddMessage(ERR_CLIENT_LOCKED_OUT, 1.0, 0.1, 0.1, 1.0)
 end
end

function TalentSetListButton_OnClick(f, button, down)
 if f.name and f.name ~= "" then
  PlaySound("igMainMenuOptionCheckBoxOn")
  TalentSetsList.selectedSetName = f.name
  -- mark the ignored slots
  TalentSetList_Update()
  TalentSetsDialogPopup:Hide()
 else
  -- This is the "New Set" button
  TalentSetsDialogPopup:Show()
  TalentSetsList.selectedSetName = nil
  TalentSetList_Update()
 end
 StaticPopup_Hide("TS_CONFIRM_DELETE_TALENT_SET")
 StaticPopup_Hide("TS_CONFIRM_OVERWRITE_TALENT_SET")
 equipment_menu_frame:Hide()
end

function TalentSetListButton_OnDoubleClick(f)
 if f.name and f.name ~= "" then
  PlaySound("igCharacterInfoTab")
  learnTalents(getSetByName(f.name))
  TalentSetsList.queuedUpdate = true
 end
end

function TalentSetListButton_OnEnter(f)
 --if f.name and f.name ~= "" then
  --GameTooltip_SetDefaultAnchor(GameTooltip, f)
 --end
end

local function equipSet_OnClick(f)
 local name = TalentSetsList.selectedSetName
 if name and name ~= "" then
  local i = getSetByName(name)
  PlaySound("igCharacterInfoTab")
  learnTalents(i)
  TalentSetsList.queuedUpdate = true
 end
end

local function saveSet_OnClick(self)
 local selectedSetName = TalentSetsList.selectedSetName
 if selectedSetName and selectedSetName ~= "" then
  local dialog = StaticPopup_Show("TS_CONFIRM_SAVE_TALENT_SET", selectedSetName)
  if dialog then
   dialog.data = selectedSetName
  else
   UIErrorsFrame:AddMessage(ERR_CLIENT_LOCKED_OUT, 1.0, 0.1, 0.1, 1.0)
  end
 end
end

local function hideFrame()
 TalentSetsMainframe:Hide()
 TalentSetsShowButton:Hide()
 equipment_menu_frame:Hide()
end

local function toggleFrameVisibility(self)
 if not PlayerTalentFramePVPTalents:IsVisible() and not PlayerTalentFrameTalents:IsVisible() then
  hideFrame()
 else
  if self == PlayerTalentFramePVPTalents then -- pvp talents
   tt = "talents_pvp"
   check = GetPvpTalentInfo
   learn = LearnPvpTalent
  else -- pve talents
   tt = "talents"
   check = GetTalentInfo
   learn = LearnTalent
  end
  specChanged()
  
  if TalentSetManager_Options.visible then
   TalentSetsMainframe:Show()
  end
  TalentSetsShowButton:Show()
 end
end

local function initializeFrame()

 local mf = CreateFrame("Frame", "TalentSetsMainframe", PlayerTalentFrame, "InsetFrameTemplate")
 mf:SetSize(200, 346)
 mf:SetPoint("LEFT", PlayerTalentFrame, "RIGHT", -1, 0)
 
 mf.title = mf:CreateFontString(nil, "BACKGROUND", "QuestFont_Shadow_Huge")
 mf.title:SetPoint("TOP", 0, -6)
 mf.title:SetText("Talent Set Manager")
 mf.title:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b)
 
 local btn = CreateFrame("Checkbutton", "TalentSetsShowButton", PlayerTalentFrame, "SpellBookSkillLineTabTemplate")
 btn.tooltip = L["talent_sets"]
 btn:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPRIGHT", 0, -20)
 btn:SetChecked(TalentSetManager_Options.visible)
 btn:SetNormalTexture("Interface\\Icons\\achievement_guildperk_ladyluck_rank2")
 
 btn:SetScript("OnClick", function()
  TalentSetManager_Options.visible = not TalentSetManager_Options.visible
  if TalentSetManager_Options.visible then
   TalentSetsMainframe:Show()
  else
   TalentSetsMainframe:Hide()
  end
 end)
 
 TalentSetsDialogPopup:SetParent(mf)
 TalentSetsDialogPopup:SetPoint("LEFT", mf, "RIGHT")
 
 local f = CreateFrame("Scrollframe", "TalentSetsList", mf, "HybridScrollFrameTemplate")
 f:SetSize(172, 304)
 f:SetPoint("TOPLEFT", mf, "TOPLEFT", 1, -30)
 --f:SetPoint("BOTTOMRIGHT", mf, "BOTTOMRIGHT", -25, 1)
 
 f:SetFrameLevel(mf:GetFrameLevel() + 1)
 
 f.scrollBar = CreateFrame("Slider", "TalentSetsFrameScrollBar", f, "HybridScrollBarTemplate")
 f.scrollBar:SetPoint("TOPLEFT", f, "TOPRIGHT", 4, -19)
 f.scrollBar:SetPoint("BOTTOMLEFT", f, "BOTTOMRIGHT", 4, 14)
 f.scrollBar.doNotHide = 1
 
 -- equip button
 f.EquipSet = CreateFrame("Button", "TalentSetsListEquipSet", f, "UIPanelButtonTemplate")
 f.EquipSet:SetSize(87, 22)
 f.EquipSet:SetPoint("TOPLEFT", f, "TOPLEFT")
 f.EquipSet:SetScript("OnClick", equipSet_OnClick)
 f.EquipSet:SetText(LEARN)
 f.EquipSet:SetFrameLevel(mf:GetFrameLevel() + 3)

 f.EquipSet.ButtonBackground = f.EquipSet:CreateTexture(nil, "BACKGROUND", 0)
 f.EquipSet.ButtonBackground:SetPoint("TOPLEFT", f, "TOPLEFT")
 f.EquipSet.ButtonBackground:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT", 0, -23)
 f.EquipSet.ButtonBackground:SetColorTexture(0, 0, 0)
 
 -- save button
 f.SaveSet = CreateFrame("Button", "TalentSetsListSaveSet", f, "UIPanelButtonTemplate")
 f.SaveSet:SetSize(87, 22)
 f.SaveSet:SetPoint("LEFT", f.EquipSet, "RIGHT")
 f.SaveSet:SetScript("OnClick", saveSet_OnClick)
 f.SaveSet:SetText(SAVE)
 f.SaveSet:SetFrameLevel(mf:GetFrameLevel() + 3)
 
 f.update = TalentSetList_Update
 f:SetScript("OnUpdate", listBox_update)
 f:SetScript("OnEvent", TalentSetList_OnEvent)
 f:SetScript("OnHide", TalentSetList_OnHide)
 f:SetScript("OnShow", TalentSetList_Update)
 
 HybridScrollFrame_CreateButtons(f, "TalentSetListButton", 2, -(f.EquipSet:GetHeight()+4))
 f:update()

 f:RegisterEvent("PLAYER_TALENT_UPDATE")
-- f:RegisterEvent("PLAYER_UPDATE_RESTING")

 mf:Hide()
end

function TalentSetsDialogPopup_OnLoad (self)
 self.buttons = {}
 
 local rows = 0
 
 local button = CreateFrame("CheckButton", "TalentSetsDialogPopupButton1", TalentSetsDialogPopup, "TalentSetPopupButtonTemplate")
 button:SetPoint("TOPLEFT", 24, -85)
 button:SetID(1)
 tinsert(self.buttons, button)
 
 local lastPos
 for i = 2, 15 do
  button = CreateFrame("CheckButton", "TalentSetsDialogPopupButton" .. i, TalentSetsDialogPopup, "TalentSetPopupButtonTemplate")
  button:SetID(i)
  
  lastPos = (i - 1) / 5
  if lastPos == math.floor(lastPos) then
   button:SetPoint("TOPLEFT", self.buttons[i - 5], "BOTTOMLEFT", 0, -8)
  else
   button:SetPoint("TOPLEFT", self.buttons[i - 1], "TOPRIGHT", 10, 0)
  end
  tinsert(self.buttons, button)
 end
 
 self.SetSelection = function(self, fTexture, Value)
  if fTexture then
   self.selectedTexture = Value
   self.selectedIcon = nil
  else
   self.selectedTexture = nil
   self.selectedIcon = Value
  end
 end
end

function TalentSetsDialogPopup_OnShow(self)
 PlaySound("igCharacterInfoOpen")
 self.name = nil
 self.id = nil
 self.isEdit = false
 RecalculateTalentSetsDialogPopup()
end

function TalentSetsDialogPopup_OnHide(self)
 TalentSetsDialogPopup.name = nil
 TalentSetsDialogPopup.id = nil
 TalentSetsDialogPopup:SetSelection(true, nil)
 TalentSetsDialogPopupEditBox:SetText("")
 icon_filenames = nil
 collectgarbage()
end

function RecalculateTalentSetsDialogPopup(name)
 local popup = TalentSetsDialogPopup
 local i = getSetByName(name)
 if i then
  TalentSetsDialogPopupEditBox:SetText(name)
  if sets[i].texture then
   popup:SetSelection(true, sets[i].texture)
  else
   popup:SetSelection(false, 1)
  end
 else
  TalentSetsDialogPopupEditBox:SetText("")
  popup:SetSelection(false, 1)
 end
 
 TalentSetsDialogPopupEditBox:HighlightText(0)

 RefreshSetIconInfo()
 local totalItems = #icon_filenames
 local texture
 if popup.selectedTexture then
  local foundIndex
  for index=1, totalItems do
   texture = icon_filenames[index]
   if texture == popup.selectedTexture then
    foundIndex = index
    break
   end
  end
  if not foundIndex then
   foundIndex = 1
  end
  
  local offsetnumIcons = floor((totalItems - 1) / 5)
  local offset = floor((foundIndex - 1) / 5)
  offset = offset + min((3 - 1), offsetnumIcons-offset) - (3 - 1)
  if foundIndex <= 15 then
   offset = 0
  end
  FauxScrollFrame_OnVerticalScroll(TalentSetsDialogPopupScrollFrame, offset * 36, 36, nil)
 else
  FauxScrollFrame_OnVerticalScroll(TalentSetsDialogPopupScrollFrame, 0, 36, nil)
 end
 TalentSetsDialogPopup_Update()
end

function RefreshSetIconInfo()
 icon_filenames = {}
 icon_filenames[1] = "INV_MISC_QUESTIONMARK"
 local index = 2
    
 for c = 1, 7 do
  for r = 1,3 do
   local _, _, texture = check(c, r, 1)

   if texture then
    texture = string.lower(texture):gsub("interface\\icons\\", "")
    icon_filenames[index] = texture
    if icon_filenames[index] then
     index = index + 1
     for j = 1, index - 1 do
      if icon_filenames[index] == icon_filenames[j] then
       icon_filenames[index] = nil
       index = index - 1
       break
      end
     end
    end
   end
  end
 end
 GetLooseMacroItemIcons(icon_filenames)
 GetLooseMacroIcons(icon_filenames)
 GetMacroItemIcons(icon_filenames)
 GetMacroIcons(icon_filenames)
end

function TalentSetsDialogPopup_Update()
 RefreshSetIconInfo()

 local popup = TalentSetsDialogPopup
 local buttons = popup.buttons
 local offset = FauxScrollFrame_GetOffset(TalentSetsDialogPopupScrollFrame) or 0
 local button
 -- Icon list
 local texture, index, button, realIndex
 for i = 1, 15 do
  local button = buttons[i]
  index = offset * 5 + i
  if index <= #icon_filenames then
   texture = icon_filenames[index]
   button.icon:SetTexture(type(texture) == "number" and texture or ("INTERFACE\\ICONS\\"..texture))
   button:Show()
   if index == popup.selectedIcon then
    button:SetChecked(true)
   elseif texture == popup.selectedTexture then
    button:SetChecked(true)
    popup:SetSelection(false, index)
   else
    button:SetChecked(false)
   end
  else
   button.icon:SetTexture("")
   button:Hide()
  end
 end
 FauxScrollFrame_Update(TalentSetsDialogPopupScrollFrame, ceil(#icon_filenames / 5) , 3, 36 )
end

function TalentSetsDialogPopupOkay_Update()
 local popup = TalentSetsDialogPopup
 local button = TalentSetsDialogPopupOkay
 
 if (popup.selectedIcon or popup.isEdit) and popup.name then
  button:Enable()
 else
  button:Disable()
 end
end

function TalentSetsDialogPopupOkay_OnClick(self, button)
 local popup = TalentSetsDialogPopup
 local iconTexture = icon_filenames[popup.selectedIcon]

 if getSetByName(popup.name) then
  if popup.isEdit and popup.name ~= popup.origName then
   UIErrorsFrame:AddMessage(L["set_already_exists"], 1.0, 0.1, 0.1, 1.0)
   return
  elseif not popup.isEdit then
   local dialog = StaticPopup_Show("TS_CONFIRM_OVERWRITE_TALENT_SET", popup.name)
   if dialog then
 	dialog.data = popup.name
 	dialog.selectedIcon = icon_filenames[popup.selectedIcon]
   else
    UIErrorsFrame:AddMessage(ERR_CLIENT_LOCKED_OUT, 1.0, 0.1, 0.1, 1.0)
   end
   return
  end
 elseif #sets >= max_talent_sets and not popup.isEdit then
  UIErrorsFrame:AddMessage(L["sets_limit_reached"], 1.0, 0.1, 0.1, 1.0)
  return
 end
 
 if popup.isEdit then
  TalentSetsList.selectedSetName = popup.name
  modifyTalentSet(popup.origName, popup.name, iconTexture)
 else
  saveTalentSet(popup.name, iconTexture)
 end
 --TalentSetsList.queuedUpdate = true
 TalentSetList_Update()
 popup:Hide()
end

function TalentSetsDialogPopupCancel_OnClick()
 TalentSetsDialogPopup:Hide()
end

function TalentSetPopupButton_OnClick(self, button)
 local popup = TalentSetsDialogPopup
 local offset = FauxScrollFrame_GetOffset(TalentSetsDialogPopupScrollFrame) or 0
 popup.selectedIcon = (offset * 5) + self:GetID()
 popup.selectedTexture = nil
 TalentSetsDialogPopup_Update()
 TalentSetsDialogPopupOkay_Update()
end

local function initialize_equipment_menu()
 equipment_menu_frame = CreateFrame("Frame", "TalentSetManagerEquipmentMenu", UIParent, "UIDropDownMenuTemplate")
 equipment_cache = {}
 equipment_menu_table = {}
 
 t_menu_empty = { text = " ", isTitle = true, notCheckable = true }
 t_menu_close = { text = CLOSE, notCheckable = true, func = function() equipment_menu_frame:Hide() end }
 
 equipment_menu_table[1] = { text = "|cffffffad"..L["equipment_menu_title1"].."|r", isTitle = true, notCheckable = true }
 equipment_menu_table[2] = { text = "|cffffffad"..L["equipment_menu_title2"].."|r", isTitle = true, notCheckable = true }
 equipment_menu_table[3] = t_menu_empty
 equipment_menu_table[4] = { text = str_no_equipment, func = equipment_menuitem_onClick }
end

local eventframe = CreateFrame("Frame")
eventframe:RegisterEvent("ADDON_LOADED")
eventframe:RegisterEvent("VARIABLES_LOADED")
eventframe:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
local function eventhandler(self, event, ...)
 if event == "VARIABLES_LOADED" then
 
  if not TalentSetManager_Options then TalentSetManager_Options = {} end
  if TalentSetManager_Options.visible == nil then TalentSetManager_Options.visible = true end
  if not TalentSetManager_Saves then TalentSetManager_Saves = {} end
  if not TalentSetManager_Saves.talents then TalentSetManager_Saves.talents = {} end
  if not TalentSetManager_Saves.talents_pvp then TalentSetManager_Saves.talents_pvp = {} end

  initialize_equipment_menu()
  
 elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
  local new_spec = GetSpecialization()
  if new_spec ~= spec then
   spec = new_spec
   specChanged()
  end
 elseif ... == "Blizzard_TalentUI" then
  spec = GetSpecialization()
  specChanged()
  
  initializeFrame()

  -- in some cases, frames are shown before the other ones are hidden, the OnHide hook is not usable
  -- wait until all the show/hide stuff is done, then check for TSM visibility
  local function delayToggle(self)
   C_Timer.After(0.01, function() toggleFrameVisibility(self) end)
  end
  
  PlayerTalentFrameTalents:HookScript("OnShow", delayToggle)
  PlayerTalentFramePVPTalents:HookScript("OnShow", delayToggle)
  PlayerTalentFrameSpecialization:HookScript("OnShow", delayToggle)
  PlayerTalentFramePetSpecialization:HookScript("OnShow", delayToggle)
 end
end
eventframe:SetScript("OnEvent", eventhandler)